//
// Copyright (c) 2016 comScore. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "SCORLiveTransmissionMode.h"
#import "SCOROfflineCacheMode.h"
#import "SCORConfigurationDelegate.h"
#import "SCORLogLevel.h"
#import "SCORUsagePropertiesAutoUpdateMode.h"


/**
 *  Default builder for client configurations. 
 *  @warning This class is not thread-safe.
 */

@interface SCORClientConfigurationBuilder : NSObject

/**
 *  The endpoint where to dispatch the live measurements.
 *  If left unchanged or set to *nil*, the default address will be used.
 */
@property(nonatomic, copy) NSString *liveEndpointURL;
/**
 *  The endpoint where to dispatch the stored measurements.
 *  If left unchanged or set to *nil*, the default address will be used.
 */
@property(nonatomic, copy) NSString *offlineFlushEndpointURL;
/**
 *  Live transmission mode.
 */
@property(nonatomic) SCORLiveTransmissionMode liveTransmissionMode;
/**
 *  Cache policy.
 */
@property(nonatomic) SCOROfflineCacheMode offlineCacheMode;
/**
 *  Preconfigured persistent labels.
 */
@property(nonatomic, copy) NSDictionary *persistentLabels;
/**
 *  Labels to send with the auto start event.
 */
@property(nonatomic, copy) NSDictionary *startLabels;
/**
 *  The preferred order for the labels.
 */

@property(nonatomic, copy) NSArray *labelOrder;
/**
 *  Application Name.
 */
@property(nonatomic, copy) NSString *applicationName;
/**
 *  Application Version.
 */
@property(nonatomic, copy) NSString *applicationVersion;
/**
 *  The auto update mode for the usage properties.
 */
@property(nonatomic) SCORUsagePropertiesAutoUpdateMode usagePropertiesAutoUpdateMode;
/**
 *  The auto update interval for the usage properties.
 */
@property(nonatomic) int usagePropertiesAutoUpdateInterval;
/**
 *  Maximum number of measurements to keep on cache.
 */
@property(nonatomic) int cacheMaxMeasurements;
/**
 *  Maximum number of batch files that the SDK can generate.
 */
@property(nonatomic) int cacheMaxBatchFiles;
/**
 *  Maximum number of cahes flushes allowed in a row.
 */
@property(nonatomic) int cacheMaxFlushesInARow;
/**
 *  Number of minutes to wait before re-trying a flush.
 */
@property(nonatomic) int cacheMinutesToRetry;
/**
 *  Number of days before the stored measurements expire.
 */
@property(nonatomic) int cacheMeasurementExpiry;
/**
 *  The interval, in seconds, to wait for every measurements flush.
 */
@property(nonatomic) long cacheFlushingInterval;
/**
 *  Enables or disables the keep alive measurements.
 */
@property(nonatomic) BOOL keepAliveMeasurement;
/**
 *  Enables or disables the secure transmission of measurements.
 */
@property(nonatomic) BOOL secureTransmission;
/**
 *  Enables or disables the tracking of uncaught exceptions.
 */
@property(nonatomic) BOOL uncaughtExceptionTracking;
/**
 *  Enables or disables VCE.
 */
@property(nonatomic) BOOL vce;


/**
 *  Sets a persistent label. If the label exist it will override it with the new value
 *  and if the value is nil the label will be removed. Persistent labels are included
 *  in all further events. Persistent labels overrides sdk labels but they can be overridden
 *  by event labels.
 *
 *  @param name NSString
 *  @param value NSString or *nil*
 */
- (void)setPersistentLabelWithName:(NSString *)name value:(NSString *)value;

/**
 *  Adds or update the given persistent labels. If any of the provided labels have a *NSNull*
 *  value, that label will be removed from the persistent labels. Existent labels that are
 *  not present on the given dictionary won't be modified.
 *
 *  @param persistentLabels NSDictionary
 */
- (void)setPersistentLabels:(NSDictionary *)persistentLabels;

/**
 *  Removes the persistent label with the given name. If the label doesn't exist this method
 *  has no effect, but will incur in a computation overhead.
 *
 *  @param name NSString
 */
- (void)removePersistentLabelWithName:(NSString *)name;

/**
 *  Removes all stored persistent labels.
 */
- (void)removeAllPersistentLabels;

/**
 * Returns the value of the specified label.
 *
 * @param name NSString
 * @return NSString or *nil*
 */
- (NSString *)persistentLabelWithName:(NSString *)name;


@end